@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Collector.pm,v 1.12 2007/06/16 16:54:08 ssinyagin Exp $
+# $Id: Collector.pm,v 1.17 2008/11/23 20:52:00 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -69,11 +69,13 @@ sub new
     foreach my $collector_type ( keys %Torrus::Collector::collectorTypes )
     {
         $self->{'types'}{$collector_type} = {};
+        $self->{'types_in_use'}{$collector_type} = 0;
     }
 
     foreach my $storage_type ( keys %Torrus::Collector::storageTypes )
     {
         $self->{'storage'}{$storage_type} = {};
+        $self->{'storage_in_use'}{$storage_type} = 0;
         
         my $storage_string = $storage_type . '-storage';
         if( ref( $Torrus::Collector::initStorage{$storage_string} ) )
@@ -107,7 +109,8 @@ sub addTarget
     $self->fetchParams($config_tree, $token, $collector_type);
 
     $self->{'targets'}{$token}{'type'} = $collector_type;
-
+    $self->{'types_in_use'}{$collector_type} = 1;
+    
     my $storage_types = $config_tree->getNodeParam($token, 'storage-type');
     foreach my $storage_type ( split( ',', $storage_types ) )
     {
@@ -126,6 +129,7 @@ sub addTarget
                   $storage_type );
             
             $self->fetchParams($config_tree, $token, $storage_string);
+            $self->{'storage_in_use'}{$storage_type} = 1;
         }
     }
 
@@ -208,6 +212,8 @@ sub fetchParams
 
     while( scalar( @maps ) > 0 )
     {
+        &Torrus::DB::checkInterrupted();
+        
         my @next_maps = ();
         foreach my $map ( @maps )
         {
@@ -261,6 +267,8 @@ sub fetchMoreParams
     my $token = shift;
     my @params = @_;
 
+    &Torrus::DB::checkInterrupted();
+
     my $ref = \$self->{'targets'}{$token}{'params'};
 
     foreach my $param ( @params )
@@ -338,6 +346,8 @@ sub deleteTarget
     my $self = shift;
     my $token = shift;
 
+    &Torrus::DB::checkInterrupted();
+
     Info('Deleting target: ' . $self->path($token));
     
     if( ref( $self->{'targets'}{$token}{'deleteProc'} ) )
@@ -390,6 +400,10 @@ sub run
 
     while( my ($collector_type, $ref) = each %{$self->{'types'}} )
     {
+        next unless $self->{'types_in_use'}{$collector_type};
+
+        &Torrus::DB::checkInterrupted();
+        
         if( $Torrus::Collector::needsConfigTree
             {$collector_type}{'runCollector'} )
         {
@@ -399,7 +413,7 @@ sub run
         }
         
         &{$Torrus::Collector::runCollector{$collector_type}}( $self, $ref );
-
+        
         if( defined( $self->{'config_tree'} ) )
         {
             undef $self->{'config_tree'};
@@ -408,6 +422,10 @@ sub run
 
     while( my ($storage_type, $ref) = each %{$self->{'storage'}} )
     {
+        next unless $self->{'storage_in_use'}{$storage_type};
+        
+        &Torrus::DB::checkInterrupted();
+        
         if( $Torrus::Collector::needsConfigTree
             {$storage_type}{'storeData'} )
         {
@@ -426,8 +444,12 @@ sub run
     
     while( my ($collector_type, $ref) = each %{$self->{'types'}} )
     {
+        next unless $self->{'types_in_use'}{$collector_type};
+        
         if( ref( $Torrus::Collector::postProcess{$collector_type} ) )
         {
+            &Torrus::DB::checkInterrupted();
+            
             if( $Torrus::Collector::needsConfigTree
                 {$collector_type}{'postProcess'} )
             {
@@ -472,7 +494,7 @@ sub setValue
                 Error('Fatal error in transformation code: ' . $@ );
                 $value = 'U';
             }
-            elsif( $value !~ /^[0-9.+-eE]+$/ and $value ne 'U' )
+            elsif( $value !~ /^[0-9.+-eE]+$/o and $value ne 'U' )
             {
                 Error('Non-numeric value after transformation: ' . $value);
                 $value = 'U';
@@ -559,6 +581,8 @@ sub beforeRun
 {
     my $self = shift;
 
+    &Torrus::DB::checkInterrupted();
+
     my $tree = $self->treeName();
     my $config_tree = new Torrus::ConfigTree(-TreeName => $tree, -Wait => 1);
     if( not defined( $config_tree ) )
@@ -579,65 +603,44 @@ sub beforeRun
     my $timestamp_key = $tree . ':' . $instance . ':collector_cache';
     my $known_ts = Torrus::TimeStamp::get( $timestamp_key );
     my $actual_ts = $config_tree->getTimestamp();
-    if( $actual_ts >= $known_ts )
+    
+    if( $actual_ts >= $known_ts or not $data->{'targets_initialized'} )
     {
-        Info("Rebuilding collector information");
+        Info('Initializing tasks for collector instance ' . $instance);
         Debug("Config TS: $actual_ts, Collector TS: $known_ts");
+        my $init_start = time();
 
-        undef $data->{'targets'};
-        $need_new_tasks = 1;
+        my $targets = {};
 
-        $data->{'db_tokens'} =
-            new Torrus::DB( 'collector_tokens' . '_' . $instance,
-                            -Subdir => $tree,
-                            -WriteAccess => 1,
-                            -Truncate    => 1 );
-        $self->cacheCollectors( $config_tree, $instance,
-                                $config_tree->token('/') );
-        # explicitly close, since we don't need it often, and sometimes
-        # open it in read-only mode
-        $data->{'db_tokens'}->closeNow();
-        undef $data->{'db_tokens'};
+        my $db_tokens = new Torrus::DB('collector_tokens' . '_' . $instance,
+                                       -Subdir => $tree);
+        
+        my $cursor = $db_tokens->cursor();
+        while( my ($token, $schedule) = $db_tokens->next($cursor) )
+        {
+            my ($period, $offset) = split(/:/o, $schedule);
+            if( not exists( $targets->{$period}{$offset} ) )
+            {
+                $targets->{$period}{$offset} = [];
+            }
+            push( @{$targets->{$period}{$offset}}, $token );
+
+            &Torrus::DB::checkInterrupted();
+        }
+        undef $cursor;
+        $db_tokens->closeNow();
+        undef $db_tokens;
+
+        &Torrus::DB::checkInterrupted();
 
         # Set the timestamp
         &Torrus::TimeStamp::setNow( $timestamp_key );
-    }
-    Torrus::TimeStamp::release();
-
-    if( not $need_new_tasks and not defined $data->{'targets'} )
-    {
-        $need_new_tasks = 1;
-
-        $data->{'db_tokens'} =
-            new Torrus::DB('collector_tokens' . '_' . $instance,
-                           -Subdir => $tree);
         
-        my $cursor = $data->{'db_tokens'}->cursor();
-        while( my ($token, $schedule) = $data->{'db_tokens'}->next($cursor) )
-        {
-            my ($period, $offset) = split(':', $schedule);
-            if( not exists( $data->{'targets'}{$period}{$offset} ) )
-            {
-                $data->{'targets'}{$period}{$offset} = [];
-            }
-            push( @{$data->{'targets'}{$period}{$offset}}, $token );
-        }
-        undef $cursor;
-        $data->{'db_tokens'}->closeNow();
-        undef $data->{'db_tokens'};
-    }
-
-    # Now fill in Scheduler's task list, if needed
-
-    if( $need_new_tasks )
-    {
-        Verbose("Initializing tasks");
-        my $init_start = time();
         $self->flushTasks();
 
-        foreach my $period ( keys %{$data->{'targets'}} )
+        foreach my $period ( keys %{$targets} )
         {
-            foreach my $offset ( keys %{$data->{'targets'}{$period}} )
+            foreach my $offset ( keys %{$targets->{$period}} )
             {
                 my $collector =
                     new Torrus::Collector( -Period => $period,
@@ -645,8 +648,9 @@ sub beforeRun
                                            -TreeName => $tree,
                                            -Instance => $instance );
 
-                foreach my $token ( @{$data->{'targets'}{$period}{$offset}} )
+                foreach my $token ( @{$targets->{$period}{$offset}} )
                 {
+                    &Torrus::DB::checkInterrupted();
                     $collector->addTarget( $config_tree, $token );
                 }
 
@@ -655,51 +659,16 @@ sub beforeRun
         }
         Verbose(sprintf("Tasks initialization finished in %d seconds",
                         time() - $init_start));
-    }
 
+        $data->{'targets_initialized'} = 1;
+    }
+    
+    Torrus::TimeStamp::release();
+    
     return 1;
 }
 
 
-sub cacheCollectors
-{
-    my $self = shift;
-    my $config_tree = shift;
-    my $instance = shift;
-    my $ptoken = shift;
-
-    my $data = $self->data();
-
-    foreach my $ctoken ( $config_tree->getChildren( $ptoken ) )
-    {
-        if( $config_tree->isSubtree( $ctoken ) )
-        {
-            $self->cacheCollectors( $config_tree, $instance, $ctoken );
-        }
-        elsif( $config_tree->isLeaf( $ctoken ) and
-               ( $config_tree->getNodeParam( $ctoken, 'ds-type' )
-                 eq 'collector' ) and
-               ( $config_tree->getNodeParam( $ctoken, 'collector-instance' )
-                 == $instance ) )
-        {
-            my $period = sprintf('%d',
-                                 $config_tree->getNodeParam
-                                 ( $ctoken, 'collector-period' ) );
-            my $offset = sprintf('%d',
-                                 $config_tree->getNodeParam
-                                 ( $ctoken, 'collector-timeoffset' ) );
-
-            $data->{'db_tokens'}->put( $ctoken, $period.':'.$offset );
-            if( not exists( $data->{'targets'}{$period}{$offset} ) )
-            {
-                $data->{'targets'}{$period}{$offset} = [];
-            }
-            push( @{$data->{'targets'}{$period}{$offset}}, $ctoken );
-        }
-    }
-}
-
-
 1;
 
 
