@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RRDStorage.pm,v 1.15 2007/06/11 15:41:18 ssinyagin Exp $
+# $Id: RRDStorage.pm,v 1.19 2008/11/23 20:52:00 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::RRDStorage;
@@ -154,6 +154,8 @@ sub storeData
     {
         while( my ($filename, $tokens) = each %{$sref->{'byfile'}} )
         {
+            &Torrus::DB::checkInterrupted();
+            
             if( not -e $filename )
             {
                 createRRD( $collector, $sref, $filename, $tokens );
@@ -166,7 +168,7 @@ sub storeData
         }
     }
 
-    undef $sref->{'values'};
+    delete $sref->{'values'};
 }
 
 
@@ -300,6 +302,8 @@ sub createRRD
     my @OPT = ( sprintf( '--start=%d', $start ),
                 sprintf( '--step=%d', $step ) );
 
+    &Torrus::DB::checkInterrupted();
+    
     Debug("Creating RRD $filename: " . join(" ", @OPT, @DS, @RRA));
 
     semaphoreDown();
@@ -344,6 +348,8 @@ sub updateRRD
                 $ref->{$1} = 1;
             }
         }
+        
+        &Torrus::DB::checkInterrupted();
     }
 
     # First we compare the sets of datasources in our memory and in RRD file
@@ -407,6 +413,8 @@ sub updateRRD
         return;
     }
 
+    &Torrus::DB::checkInterrupted();
+
     # Build the arguments for RRDs::update.
     my $template;
     my $values;
@@ -465,6 +473,8 @@ sub updateRRD
     my @cmd = ( "--template=" . $template,
                 sprintf("%d%s", $avg_ts, $values) );
 
+    &Torrus::DB::checkInterrupted();
+
     if( $threadsInUse )
     {
         # Process errors from RRD update thread
@@ -500,12 +510,17 @@ sub updateRRD
 # A background thread that updates RRD files
 sub rrdUpdateThread
 {
+    &Torrus::DB::setSafeSignalHandlers();
+    $| = 1;
     &Torrus::Log::setTID( threads->tid() );
+    
     my $cmdlist;
     &threads::shared::share( \$cmdlist );
     
     while(1)
     {
+        &Torrus::DB::checkInterrupted();
+        
         $cmdlist = $thrUpdateQueue->dequeue();
         
         if( isDebug )
