@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: SNMP.pm,v 1.38 2007/07/27 15:56:00 ssinyagin Exp $
+# $Id: SNMP.pm,v 1.49 2008/11/19 21:23:35 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::SNMP;
@@ -35,6 +35,8 @@ $Torrus::Collector::collectorTypes{'snmp'} = 1;
 # List of needed parameters and default values
 
 $Torrus::Collector::params{'snmp'} = {
+    'snmp-ipversion'    => undef,
+    'snmp-transport'    => undef,
     'snmp-version'      => undef,
     'snmp-port'         => undef,
     'snmp-community'    => undef,
@@ -54,7 +56,9 @@ $Torrus::Collector::params{'snmp'} = {
     'snmp-object'       => undef,
     'snmp-oids-per-pdu' => undef,
     'snmp-object-type'  => 'OTHER',
-    'snmp-check-sysuptime' => 'yes'
+    'snmp-check-sysuptime' => 'yes',
+    'snmp-max-msg-size' => undef,
+    'snmp-ignore-mib-errors' => undef,
     };
 
 my $sysUpTime = '1.3.6.1.2.1.1.3.0';
@@ -112,13 +116,13 @@ sub initTarget
     $collector->registerDeleteCallback
         ( $token, \&Torrus::Collector::SNMP::deleteTarget );
 
-    my $ipaddr = getHostIpAddress( $collector, $token );
-    if( not defined( $ipaddr ) )
+    my $hostname = getHostname( $collector, $token );
+    if( not defined( $hostname ) )
     {
         return 0;
     }
 
-    $tref->{'ipaddr'} = $ipaddr;
+    $tref->{'hostname'} = $hostname;
     
     return Torrus::Collector::SNMP::initTargetAttributes( $collector, $token );
 }
@@ -129,10 +133,12 @@ sub initTargetAttributes
     my $collector = shift;
     my $token = shift;
 
+    &Torrus::DB::checkInterrupted();
+
     my $tref = $collector->tokenData( $token );
     my $cref = $collector->collectorData( 'snmp' );
 
-    my $ipaddr = $tref->{'ipaddr'};
+    my $hostname = $tref->{'hostname'};
     my $port = $collector->param($token, 'snmp-port');
     my $version = $collector->param($token, 'snmp-version');
 
@@ -148,7 +154,7 @@ sub initTargetAttributes
         $community = $collector->param($token, 'snmp-username');
     }
 
-    my $hosthash = join('|', $ipaddr, $port, $community);
+    my $hosthash = join('|', $hostname, $port, $community);
     $tref->{'hosthash'} = $hosthash;
 
     if( $version eq '1' )
@@ -213,12 +219,17 @@ sub initTargetAttributes
     {
         $cref->{'nosysuptime'}{$hosthash} = 1;
     }
+
+    if( $collector->param($token, 'snmp-ignore-mib-errors') eq 'yes' )
+    {
+        $cref->{'ignoremiberrors'}{$hosthash}{$oid} = 1;
+    }
     
     return 1;
 }
 
 
-sub getHostIpAddress
+sub getHostname
 {
     my $collector = shift;
     my $token = shift;
@@ -227,41 +238,15 @@ sub getHostIpAddress
 
     my $hostname = $collector->param($token, 'snmp-host');
     my $domain = $collector->param($token, 'domain-name');
-    if( $hostname !~ /\./o and length( $domain ) > 0 )
+    
+    if( length( $domain ) > 0 and
+        index($hostname, '.') < 0 and
+        index($hostname, ':') < 0 )
     {
         $hostname .= '.' . $domain;
     }
-
-    my $ipaddr;
-    if( $hostname =~ /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/o )
-    {
-        $ipaddr = $hostname;
-    }
-    else
-    {
-        if( not defined( $cref->{'dnscache'}{$hostname} ) )
-        {
-            my $h = gethost($hostname);
-
-            if( not $h )
-            {
-                Error("Cannot resolve $hostname in " .
-                      $collector->path($token));
-                $collector->deleteTarget($token);
-                return undef;
-            }
-
-            # Save the resolved address
-            $ipaddr = inet_ntoa( $h->addr() );
-            $cref->{'dnscache'}{$hostname} = $ipaddr;
-        }
-        else
-        {
-            $ipaddr = $cref->{'dnscache'}{$hostname};
-        }
-    }
-
-    return $ipaddr;
+    
+    return $hostname;
 }
 
 
@@ -277,10 +262,14 @@ sub snmpSessionArgs
         return $cref->{'snmpargs'}{$hosthash};
     }
 
-    my ($ipaddr, $port, $community) = split(/\|/o, $hosthash);
+    my $transport = $collector->param($token, 'snmp-transport') . '/ipv' .
+        $collector->param($token, 'snmp-ipversion');
+    
+    my ($hostname, $port, $community) = split(/\|/o, $hosthash);
 
     my $version = $collector->param($token, 'snmp-version');
-    my $ret = [ -hostname     => $ipaddr,
+    my $ret = [ -domain       => $transport,
+                -hostname     => $hostname,
                 -port         => $port,
                 -timeout      => $collector->param($token, 'snmp-timeout'),
                 -retries      => $collector->param($token, 'snmp-retries'),
@@ -334,7 +323,15 @@ sub openBlockingSession
     {
         Error('Cannot create SNMP session for ' . $hosthash . ': ' . $error);
     }
-
+    else
+    {
+        my $maxmsgsize = $collector->param($token, 'snmp-max-msg-size');
+        if( defined( $maxmsgsize ) and $maxmsgsize > 0 )
+        {
+            $session->max_msg_size( $maxmsgsize );
+        }
+    }
+    
     return $session;
 }
 
@@ -378,8 +375,14 @@ sub openNonblockingSession
                 Debug('Set SO_RCVBUF to ' . $buflen);
             }
         }
-    }
 
+        my $maxmsgsize = $collector->param($token, 'snmp-max-msg-size');
+        if( defined( $maxmsgsize ) and $maxmsgsize > 0 )
+        {
+            $session->max_msg_size( $maxmsgsize );
+        }
+    }
+    
     return $session;
 }
 
@@ -513,7 +516,15 @@ sub lookupMap
         {
             return undef;
         }
-        
+
+        if( scalar(@mappingSessions) >=
+            $Torrus::Collector::SNMP::maxSessionsPerDispatcher )
+        {
+            snmp_dispatcher();
+            @mappingSessions = ();
+            %mapLookupScheduled = ();
+        }
+
         # Retrieve map from host
         Debug("Retrieving map $map from $hosthash");
 
@@ -588,17 +599,19 @@ sub mapLookupCallback
     my $hosthash = shift;
     my $map = shift;
 
+    &Torrus::DB::checkInterrupted();
+    
     Debug('Received mapping PDU from ' . $hosthash);
 
     my $result = $session->var_bind_list();
     if( defined $result )
     {
-        while( my( $val, $key ) = each %{$result} )
+        my $preflen = length($map) + 1;
+        
+        while( my( $oid, $key ) = each %{$result} )
         {
-            my $quoted = quotemeta( $map );
-            $val =~ s/^$quoted\.//;
-            $maps{$hosthash}{$map}{$key} =
-                $val;
+            my $val = substr($oid, $preflen);
+            $maps{$hosthash}{$map}{$key} = $val;
             # Debug("Map $map discovered: '$key' -> '$val'");
         }
     }
@@ -781,112 +794,145 @@ sub runCollector
 {
     my $collector = shift;
     my $cref = shift;
-        
+
+    # Info(sprintf('runCollector() Offset: %d, active hosts: %d, maps: %d',
+    #              $collector->offset(),
+    #              scalar( keys %{$cref->{'activehosts'}} ),
+    #              scalar(keys %maps)));
+    
     # Create one SNMP session per host address.
     # We assume that version, timeout and retries are the same
     # within one address
 
-    my @sessions;
+    # We limit the number of sessions per snmp_dispatcher run
+    # because of some strange bugs: with more than 400 sessions per
+    # dispatcher, some requests are not sent out
 
-    foreach my $hosthash ( keys %{$cref->{'activehosts'}} )
+    my @hosts = keys %{$cref->{'activehosts'}};
+    
+    while( scalar(@mappingSessions) + scalar(@hosts) > 0 )
     {
-        my @oids = sort keys %{$cref->{'targets'}{$hosthash}};
-        
-        # Find one representative token for the host
-        
-        if( scalar( @oids ) == 0 )
+        my @batch = ();
+        while( ( scalar(@mappingSessions) + scalar(@batch) <
+                 $Torrus::Collector::SNMP::maxSessionsPerDispatcher )
+               and
+               scalar(@hosts) > 0 )
         {
-            next;
+            push( @batch, pop( @hosts ) );
         }
-        
-        my @reptokens = keys %{$cref->{'targets'}{$hosthash}{$oids[0]}};
-        if( scalar( @reptokens ) == 0 )
-        {
-            next;
-        }
-        my $reptoken = $reptokens[0];
-                    
-        my $session =
-            openNonblockingSession( $collector, $reptoken, $hosthash );
-        
-        if( not defined($session) )
-        {
-            return 0;
-        }
-        else
-        {
-            Debug('Created SNMP session for ' . $hosthash);
-            push( @sessions, $session );
-        }
-        
-        my $oids_per_pdu = $cref->{'oids_per_pdu'}{$hosthash};
 
-        my @pdu_oids = ();
-        my $delay = 0;
-        
-        while( scalar( @oids ) > 0 )
-        {
-            my $oid = shift @oids;
-            push( @pdu_oids, $oid );
+        &Torrus::DB::checkInterrupted();
 
-            if( scalar( @oids ) == 0 or
-                ( scalar( @pdu_oids ) >= $oids_per_pdu ) )
+        my @sessions;
+
+        foreach my $hosthash ( @batch )
+        {
+            my @oids = sort keys %{$cref->{'targets'}{$hosthash}};
+
+            # Info(sprintf('Host %s: %d OIDs',
+            #              $hosthash,
+            #              scalar(@oids)));
+            
+            # Find one representative token for the host
+            
+            if( scalar( @oids ) == 0 )
             {
-                if( not $cref->{'nosysuptime'}{$hosthash} )
-                {
-                    # We insert sysUpTime into every PDU, because
-                    # we need it in further processing
-                    push( @pdu_oids, $sysUpTime );
-                }
-                
-                if( Torrus::Log::isDebug() )
-                {
-                    Debug("Sending SNMP PDU to $hosthash:");
-                    foreach my $oid ( @pdu_oids )
-                    {
-                        Debug($oid);
-                    }
-                }
+                next;
+            }
+        
+            my @reptokens = keys %{$cref->{'targets'}{$hosthash}{$oids[0]}};
+            if( scalar( @reptokens ) == 0 )
+            {
+                next;
+            }
+            my $reptoken = $reptokens[0];
+            
+            my $session =
+                openNonblockingSession( $collector, $reptoken, $hosthash );
+            
+            &Torrus::DB::checkInterrupted();
+            
+            if( not defined($session) )
+            {
+                return 0;
+            }
+            else
+            {
+                Debug('Created SNMP session for ' . $hosthash);
+                push( @sessions, $session );
+            }
+            
+            my $oids_per_pdu = $cref->{'oids_per_pdu'}{$hosthash};
 
-                # Generate the list of tokens that form this PDU
-                my $pdu_tokens = {};
-                foreach my $oid ( @pdu_oids )
+            my @pdu_oids = ();
+            my $delay = 0;
+            
+            while( scalar( @oids ) > 0 )
+            {
+                my $oid = shift @oids;
+                push( @pdu_oids, $oid );
+
+                if( scalar( @oids ) == 0 or
+                    ( scalar( @pdu_oids ) >= $oids_per_pdu ) )
                 {
-                    if( defined( $cref->{'targets'}{$hosthash}{$oid} ) )
+                    if( not $cref->{'nosysuptime'}{$hosthash} )
                     {
-                        foreach my $token
-                            ( keys %{$cref->{'targets'}{$hosthash}{$oid}} )
+                        # We insert sysUpTime into every PDU, because
+                        # we need it in further processing
+                        push( @pdu_oids, $sysUpTime );
+                    }
+                    
+                    if( Torrus::Log::isDebug() )
+                    {
+                        Debug("Sending SNMP PDU to $hosthash:");
+                        foreach my $oid ( @pdu_oids )
                         {
-                            $pdu_tokens->{$oid}{$token} = 1;
+                            Debug($oid);
                         }
                     }
+
+                    # Generate the list of tokens that form this PDU
+                    my $pdu_tokens = {};
+                    foreach my $oid ( @pdu_oids )
+                    {
+                        if( defined( $cref->{'targets'}{$hosthash}{$oid} ) )
+                        {
+                            foreach my $token
+                                ( keys %{$cref->{'targets'}{$hosthash}{$oid}} )
+                            {
+                                $pdu_tokens->{$oid}{$token} = 1;
+                            }
+                        }
+                    }
+                    my $result =
+                        $session->
+                        get_request( -delay => $delay,
+                                     -callback =>
+                                     [ \&Torrus::Collector::SNMP::callback,
+                                       $collector, $pdu_tokens, $hosthash ],
+                                     -varbindlist => \@pdu_oids );
+                    if( not defined $result )
+                    {
+                        Error("Cannot create SNMP request: " .
+                              $session->error);
+                    }
+                    @pdu_oids = ();
+                    $delay += 0.01;
                 }
-                my $result =
-                    $session->
-                    get_request( -delay => $delay,
-                                 -callback =>
-                                 [ \&Torrus::Collector::SNMP::callback,
-                                   $collector, $pdu_tokens, $hosthash ],
-                                 -varbindlist => \@pdu_oids );
-                if( not defined $result )
-                {
-                    Error("Cannot create SNMP request: " .
-                          $session->error);
-                }
-                @pdu_oids = ();
-                $delay += 0.01;
             }
         }
-    }
-    
-    snmp_dispatcher();
+        
+        &Torrus::DB::checkInterrupted();
+        
+        snmp_dispatcher();
 
-    # Check if there were pending map lookup sessions
-    
-    if( scalar( @mappingSessions ) > 0 )
-    {
-        @mappingSessions = ();
-        %mapLookupScheduled = ();
+        # Check if there were pending map lookup sessions
+        
+        if( scalar( @mappingSessions ) > 0 )
+        {
+            @mappingSessions = ();
+            %mapLookupScheduled = ();
+        }
     }
 }
 
@@ -898,6 +944,8 @@ sub callback
     my $pdu_tokens = shift;
     my $hosthash = shift;
 
+    &Torrus::DB::checkInterrupted();
+    
     my $cref = $collector->collectorData( 'snmp' );
 
     Debug('SNMP Callback executed for ' . $hosthash);
@@ -981,11 +1029,14 @@ sub callback
                 $value eq 'noSuchInstance' or
                 $value eq 'endOfMibView' )
             {
-                Error("Error retrieving $oid from $hosthash: $value");
-                
-                foreach my $token ( keys %{$pdu_tokens->{$oid}} )
+                if( not $cref->{'ignoremiberrors'}{$hosthash}{$oid} )
                 {
-                    $collector->deleteTarget($token);
+                    Error("Error retrieving $oid from $hosthash: $value");
+                    
+                    foreach my $token ( keys %{$pdu_tokens->{$oid}} )
+                    {
+                        $collector->deleteTarget($token);
+                    }
                 }
             }
             else
@@ -1055,7 +1106,9 @@ sub postProcess
         while( my ( $maphash, $expire ) = each %mapsExpire )
         {
             if( $expire <= $now and not $mapLookupScheduled{$maphash} )
-            {                
+            {
+                &Torrus::DB::checkInterrupted();
+
                 my ( $hosthash, $map ) = split( /\#/o, $maphash );
 
                 if( $unreachableHostDeleted{$hosthash} )
@@ -1095,6 +1148,8 @@ sub postProcess
     
     foreach my $token ( keys %{$cref->{'needsRemapping'}} )
     {
+        &Torrus::DB::checkInterrupted();
+
         delete $cref->{'needsRemapping'}{$token};
         if( not Torrus::Collector::SNMP::initTargetAttributes
             ( $collector, $token ) )
