@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: SNMP_Params.pm,v 1.5 2007/04/10 14:28:34 ssinyagin Exp $
+# $Id: SNMP_Params.pm,v 1.8 2008/06/07 21:19:32 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::SNMP_Params;
@@ -24,6 +24,8 @@ package Torrus::Collector::SNMP_Params;
 
 my %validatorLeafParams =
     (
+     'snmp-ipversion'     => {'4'   => undef, '6'   => undef},
+     'snmp-transport'     => {'udp' => undef, 'tcp' => undef},
      'snmp-host'          => undef,
      'snmp-port'          => undef,
      '+snmp-localaddr'    => undef,
@@ -51,7 +53,9 @@ my %validatorLeafParams =
      '+snmp-object-type'  => { 'OTHER'     => undef,
                                'COUNTER64' => undef },
      '+snmp-check-sysuptime' => { 'yes' => undef,
-                                   'no'  => undef }
+                                   'no'  => undef },
+     '+snmp-max-msg-size' => undef,
+     '+snmp-ignore-mib-errors' => undef,
      );
 
 sub initValidatorLeafParams
@@ -64,6 +68,8 @@ sub initValidatorLeafParams
 
 my %admInfoLeafParams =
     (
+     'snmp-ipversion'     => undef,
+     'snmp-transport'     => undef,
      'snmp-host'          => undef,
      'snmp-port'          => undef,
      'snmp-localaddr'     => undef,
@@ -83,12 +89,16 @@ my %admInfoLeafParams =
      'snmp-retries'       => undef,
      'snmp-oids-per-pdu'  => undef,
      'snmp-object-type'   => undef,
-     'snmp-check-sysuptime' => undef
+     'snmp-check-sysuptime' => undef,
+     'snmp-max-msg-size' => undef,
+     'snmp-ignore-mib-errors' => undef,
      );
 
 
 my %admInfoParamCategories =
     (
+     'snmp-ipversion'     => 'SNMP',
+     'snmp-transport'     => 'SNMP',
      'snmp-host'          => 'SNMP',
      'snmp-port'          => 'SNMP',
      'snmp-localaddr'     => 'SNMP',
@@ -108,7 +118,9 @@ my %admInfoParamCategories =
      'snmp-retries'       => 'SNMP',
      'snmp-oids-per-pdu'  => 'SNMP',
      'snmp-object-type'   => 'SNMP',
-     'snmp-check-sysuptime' => 'SNMP'
+     'snmp-check-sysuptime' => 'SNMP',
+     'snmp-max-msg-size'  => 'SNMP',
+     'snmp-ignore-mib-errors' => 'SNMP'
      );
 
 
