@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Validator.pm,v 1.10 2007/06/21 08:00:15 ssinyagin Exp $
+# $Id: Validator.pm,v 1.12 2008/09/25 21:24:46 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -164,6 +164,9 @@ sub validateNode
 {
     my $config_tree = shift;
     my $token = shift;
+
+    &Torrus::DB::checkInterrupted();
+
     my $ok = 1;
 
     if( $config_tree->isLeaf($token) )
@@ -392,7 +395,8 @@ sub validateNode
                 }
             }
         }
-        elsif($config_tree->getNodeParam( $token, 'leaf-type' ) eq 'rrd-cdef')
+        elsif( $dsType eq 'rrd-file' and
+               $config_tree->getNodeParam( $token, 'leaf-type' ) eq 'rrd-cdef')
         {
             my $expr = $config_tree->getNodeParam( $token, 'rpn-expr' );
             if( defined( $expr ) )
@@ -467,6 +471,8 @@ sub validateRPN
     my $config_tree = shift;
     my $timeoffset_supported = shift;
 
+    &Torrus::DB::checkInterrupted();
+    
     my $ok = 1;
 
     # There must be at least one DS reference
@@ -550,6 +556,8 @@ sub validateViews
 
     foreach my $view ($config_tree->getViewNames())
     {
+        &Torrus::DB::checkInterrupted();
+        
         $ok = validateInstanceParams($config_tree, $view,
                                      'view', \%view_params) ? $ok:0;
         if( $ok and $config_tree->getParam($view, 'view-type') eq 'rrgraph' )
@@ -693,7 +701,7 @@ sub validateMonitors
     my $ok = 1;
 
     foreach my $action ($config_tree->getActionNames())
-    {
+    {        
         $ok = validateInstanceParams($config_tree, $action,
                                      'action', \%action_params) ? $ok:0;
         my $atype = $config_tree->getParam($action, 'action-type');
@@ -817,6 +825,8 @@ sub validateTokensets
 
     foreach my $tset ($config_tree->getTsets())
     {
+        &Torrus::DB::checkInterrupted();
+        
         $view = $config_tree->getParam($tset, 'default-tset-view');
         if( not defined( $view ) )
         {
@@ -847,6 +857,8 @@ sub validateInstanceParams
     my $inst_type = shift;
     my $mapref = shift;
 
+    &Torrus::DB::checkInterrupted();
+    
     # Debug("Validating $inst_type $inst_name");
 
     my $ok = 1;
