@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Writer.pm,v 1.15 2007/06/21 08:00:19 ssinyagin Exp $
+# $Id: Writer.pm,v 1.19 2008/09/25 21:24:47 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 #
@@ -43,6 +43,9 @@ our %multigraph_remove_space =
 # instance of Torrus::ServiceID object, if needed
 my $srvIdParams;
 
+# tree names where we initialized service IDs
+my %srvIdInitialized;
+
 
 sub new
 {
@@ -64,7 +67,17 @@ sub new
 
     $self->{'collectorInstances'} =
         Torrus::SiteConfig::collectorInstances( $self->treeName() );
-    
+
+    $self->{'db_collectortokens'} = [];
+    foreach my $instance ( 0 .. ($self->{'collectorInstances'} - 1) )
+    {
+        $self->{'db_collectortokens'}->[$instance] =
+            new Torrus::DB( 'collector_tokens' . '_' . $instance,
+                            -Subdir => $self->treeName(),
+                            -WriteAccess => 1,
+                            -Truncate    => 1 );
+    }
+        
     return $self;
 }
 
@@ -368,6 +381,8 @@ sub postProcess
     $self->{'viewParamsProcessed'} = {};
     foreach my $vname ( $self->getViewNames() )
     {
+        &Torrus::DB::checkInterrupted();
+        
         $self->propagateViewParams( $vname );
     }
     return $ok;
@@ -378,6 +393,8 @@ sub postProcessNodes
     my $self = shift;
     my $token = shift;
 
+    &Torrus::DB::checkInterrupted();
+
     my $ok = 1;
 
     if( not defined( $token ) )
@@ -544,37 +561,80 @@ sub postProcessNodes
                                          $newOffset );
                 }
 
+                $self->{'db_collectortokens'}->[$instance]->put
+                    ( $token, sprintf('%d:%d', $period, $newOffset) );
+
                 my $storagetypes =
                     $self->getNodeParam( $token, 'storage-type' );
                 foreach my $stype ( split(/,/o, $storagetypes) )
                 {
                     if( $stype eq 'ext' )
                     {
-                        my $tree = $self->treeName();
                         if( not defined( $srvIdParams ) )
                         {
                             $srvIdParams =
                                 new Torrus::ServiceID( -WriteAccess => 1 );
-                            $srvIdParams->cleanAllForTree( $tree );
                         }
 
+                        my $srvTrees =
+                            $self->getNodeParam($token, 'ext-service-trees');
+
+                        if( not defined( $srvTrees ) or
+                            length( $srvTrees ) == 0 )
+                        {
+                            $srvTrees = $self->treeName();
+                        }
+                                                
                         my $serviceid =
                             $self->getNodeParam($token, 'ext-service-id');
-                        if( $srvIdParams->idExists( $serviceid ) )
-                        {
-                            Error('Duplicate ServiceID: ' . $serviceid);
-                            $ok = 0;
-                        }
-                        else
-                        {
-                            my $params = {
-                                'tree' => $tree,
-                                'token' => $token,
-                                'dstype' => $self->getNodeParam($token,
-                                                                'ext-dstype'),
-                                'units' => $self->getNodeParam
-                                    ($token, 'ext-service-units') };
 
+                        foreach my $srvTree (split(/\s*,\s*/o, $srvTrees))
+                        {
+                            if( not Torrus::SiteConfig::treeExists($srvTree) )
+                            {
+                                Error
+                                    ('Error processing ext-service-trees' .
+                                     'for ' . $self->path( $token ) .
+                                     ': tree ' . $srvTree .
+                                     ' does not exist');
+                                $ok = 0;
+                            }
+                            else
+                            {
+                                if( not $srvIdInitialized{$srvTree} )
+                                {
+                                    $srvIdParams->cleanAllForTree
+                                        ( $srvTree );
+                                    $srvIdInitialized{$srvTree} = 1;
+                                }
+                                else
+                                {
+                                    if( $srvIdParams->idExists( $serviceid,
+                                                                $srvTree ) )
+                                    {
+                                        Error('Duplicate ServiceID: ' .
+                                              $serviceid . ' in tree ' .
+                                              $srvTree);
+                                        $ok = 0;
+                                    }
+                                }
+                            }
+                        }
+
+                        if( $ok )
+                        {
+                            # sorry for ackward Emacs auto-indent
+                            my $params = {
+                                'trees' => $srvTrees,
+                                'token' => $token,
+                                'dstype' =>
+                                    $self->getNodeParam($token,
+                                                        'ext-dstype'),
+                                    'units' =>
+                                    $self->getNodeParam
+                                    ($token, 'ext-service-units')
+                                };
+                            
                             $srvIdParams->add( $serviceid, $params );
                         }
                     }
