@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: DB.pm,v 1.6 2007/04/11 10:25:13 ssinyagin Exp $
+# $Id: DB.pm,v 1.9 2008/11/23 20:52:00 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::DB;
@@ -176,6 +176,109 @@ sub new
 }
 
 
+# It is strongly inadvisable to do anything inside a signal handler when DB
+# operation is in progress
+
+our $interrupted = 0;
+
+my $signalHandlersSet = 0;
+my $safeSignals = 0;
+
+
+
+
+
+sub setSignalHandlers
+{
+    if( $signalHandlersSet )
+    {
+        return;
+    }
+    
+    $SIG{'TERM'} = sub {
+        if( $safeSignals )
+        {
+            Warn('Received SIGTERM. Scheduling to exit.');
+            $interrupted = 1;
+        }
+        else
+        {
+            Warn('Received SIGTERM. Stopping the process.');
+            exit(1);
+        }            
+    };
+
+    $SIG{'INT'} = sub {
+        if( $safeSignals )
+        {
+            Warn('Received SIGINT. Scheduling to exit.');
+            $interrupted = 1;
+        }
+        else
+        {
+            Warn('Received SIGINT. Stopping the process');
+            exit(1);
+        }            
+    };
+    
+
+    $SIG{'PIPE'} = sub {
+        if( $safeSignals )
+        {
+            Warn('Received SIGPIPE. Scheduling to exit.');
+            $interrupted = 1;
+        }
+        else
+        {
+            Warn('Received SIGPIPE. Stopping the process');
+            exit(1);
+        }            
+    };
+    
+    $SIG{'QUIT'} = sub {
+        if( $safeSignals )
+        {
+            Warn('Received SIGQUIT. Scheduling to exit.');
+            $interrupted = 1;
+        }
+        else
+        {
+            Warn('Received SIGQUIT. Stopping the process');
+            exit(1);
+        }            
+    };
+
+    $signalHandlersSet = 1;
+}
+
+
+sub setSafeSignalHandlers
+{
+    setSignalHandlers();
+    $safeSignals = 1;
+}
+
+
+sub setUnsafeSignalHandlers
+{
+    setSignalHandlers();
+    $safeSignals = 0;
+}
+    
+
+# If we were previously interrupted, gracefully exit now
+
+sub checkInterrupted
+{
+    if( $interrupted )
+    {
+        Warn('Stopping the process');
+        exit(1);
+    }
+}
+
+
+
 sub closeNow
 {
     my $self = shift;
@@ -199,7 +302,8 @@ sub cleanupEnvironment
         }
         
         Debug("Destroying BerkeleyDB::Env");
-        undef $Torrus::DB::env;
+        $Torrus::DB::env->close();
+        $Torrus::DB::env = undef;
 
         if( -z $Torrus::DB::dbEnvErrFile )
         {
@@ -208,6 +312,7 @@ sub cleanupEnvironment
     }
 }
 
+
 sub trunc
 {
     my $self = shift;
@@ -584,6 +689,17 @@ sub delFromList
 }
 
 
+
+sub deleteList
+{
+    my $self = shift;
+    my $key = shift;
+
+    delete $self->{'listcache'}{$key};
+    $self->del($key);
+}
+
+
 1;
 
 
