@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: DevDiscover.pm,v 1.29 2007/05/04 11:08:04 ssinyagin Exp $
+# $Id: DevDiscover.pm,v 1.34 2008/06/15 19:21:07 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Core SNMP device discovery module
@@ -62,6 +62,8 @@ $defaultParams{'domain-name'} = '';
 $defaultParams{'host-subtree'} = '';
 $defaultParams{'snmp-check-sysuptime'} = 'yes';
 $defaultParams{'show-recursive'} = 'yes';
+$defaultParams{'snmp-ipversion'} = '4';
+$defaultParams{'snmp-transport'} = 'udp';
 
 our @copyParams =
     ( 'collector-period',
@@ -79,6 +81,8 @@ our @copyParams =
       'snmp-port',
       'snmp-localaddr',
       'snmp-localport',
+      'snmp-ipversion',
+      'snmp-transport',
       'snmp-community',
       'snmp-version',
       'snmp-username',
@@ -92,6 +96,7 @@ our @copyParams =
       'snmp-retries',
       'snmp-oids-per-pdu',
       'snmp-check-sysuptime',
+      'snmp-max-msg-size',
       'system-id' );
 
 
@@ -169,6 +174,7 @@ sub discover
 
     my %snmpargs;
     my $community;
+    
     my $version = $devdetails->param( 'snmp-version' );
     $snmpargs{'-version'} = $version;    
 
@@ -180,6 +186,9 @@ sub discover
         }
     }
     
+    $snmpargs{'-domain'} = $devdetails->param('snmp-transport') . '/ipv' .
+        $devdetails->param('snmp-ipversion');
+
     if( $version eq '1' or $version eq '2c' )
     {
         $community = $devdetails->param( 'snmp-community' );
@@ -216,7 +225,7 @@ sub discover
     my $hostname = $devdetails->param('snmp-host');
     my $domain = $devdetails->param('domain-name');
 
-    if( $domain and index($hostname, '.') < 0 )
+    if( $domain and index($hostname, '.') < 0 and index($hostname, ':') < 0 )
     {
          $hostname .= '.' . $domain;
     }
@@ -332,6 +341,17 @@ sub discover
         }
     }
 
+    # If snmp-host is ipv6 address, system-id needs to be adapted to
+    # remove colons
+    
+    if( not defined( $data->{'param'}{'system-id'} ) and
+        index($data->{'param'}{'snmp-host'}, ':') >= 0 )
+    {
+        my $systemid = $data->{'param'}{'snmp-host'};
+        $systemid =~ s/:/_/g;
+        $data->{'param'}{'system-id'} = $systemid;
+    }
+
     if( not defined( $devdetails->snmpVar($self->oiddef('sysUpTime')) ) )
     {
         Debug('Agent does not support sysUpTime');
@@ -499,7 +519,7 @@ sub buildConfig
                 push( @{$data->{'templates'}}, '::holt-winters-defaults' );
             }
 
-
+            
             my $devNodeName = $devdetails->param('symbolic-name');
             if( length( $devNodeName ) == 0 )
             {
@@ -508,8 +528,8 @@ sub buildConfig
                 {
                     $devNodeName = $devdetails->param('snmp-host');
                 }
-            }
-
+            }                
+                
             my $devNode = $cb->addSubtree( $subtreeNode, $devNodeName,
                                            $data->{'param'},
                                            $data->{'templates'} );
@@ -523,6 +543,16 @@ sub buildConfig
                 }
             }
 
+            my $includeFiles = $devdetails->param('include-files');
+            if( length( $includeFiles ) > 0 )
+            {
+                foreach my $file ( split( '\s*,\s*', $includeFiles ) )
+                {
+                    $cb->addFileInclusion( $file );
+                }
+            }
+                    
+
             # Let the device type-specific modules add children
             # to the subtree
             foreach my $devtype
@@ -803,7 +833,7 @@ sub storeSnmpVars
 
     while( my( $oid, $value ) = each %{$vars} )
     {
-        if( $oid !~ /^\d[0-9.]+\d$/ )
+        if( $oid !~ /^\d[0-9.]+\d$/o )
         {
             Error("Invalid OID syntax: '$oid'");
         }
@@ -813,8 +843,8 @@ sub storeSnmpVars
             
             while( length( $oid ) > 0 )
             {
-                $oid =~ s/\d+$//;
-                $oid =~ s/\.$//;
+                $oid =~ s/\d+$//o;
+                $oid =~ s/\.$//o;
                 if( not exists( $self->{'snmpvars'}{$oid} ) )
                 {
                     $self->{'snmpvars'}{$oid} = undef;
@@ -822,6 +852,9 @@ sub storeSnmpVars
             }
         }
     }
+
+    # Clean the cache of sorted OIDs
+    $self->{'sortedoids'} = undef;
 }
 
 ##
@@ -858,18 +891,28 @@ sub getSnmpIndices
     my $self = shift;
     my $prefix = shift;
 
+    # Remember the sorted OIDs, as sorting is quite expensive for large
+    # arrays.
+    
+    if( not defined( $self->{'sortedoids'} ) )
+    {
+        $self->{'sortedoids'} = [];
+        push( @{$self->{'sortedoids'}},
+              Net::SNMP::oid_lex_sort( keys %{$self->{'snmpvars'}} ) );
+    }
+        
     my @ret;
-    my $prefixLen = length( $prefix );
+    my $prefixLen = length( $prefix ) + 1;
     my $matched = 0;
 
-    foreach my $oid ( Net::SNMP::oid_lex_sort( keys %{$self->{'snmpvars'}} ) )
+    foreach my $oid ( @{$self->{'sortedoids'}} )
     {
         if( defined($self->{'snmpvars'}{$oid} ) )
         {
             if( Net::SNMP::oid_base_match( $prefix, $oid ) )
             {
                 # Extract the index from OID
-                my $index = substr( $oid, $prefixLen + 1 );
+                my $index = substr( $oid, $prefixLen );
                 push( @ret, $index );
                 $matched = 1;
             }
