@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoGeneric.pm,v 1.9 2006/02/13 14:10:32 ssinyagin Exp $
+# $Id: CiscoGeneric.pm,v 1.10 2008/06/02 23:28:02 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Common Cisco MIBs, supported by many IOS and CatOS devices
@@ -397,7 +397,7 @@ sub buildConfig
                      'precedence' => -600};
         my $templates = [];
                 
-        $param->{'data-file'} = '%snmp-host%_power.rrd';
+        $param->{'data-file'} = '%system-id%_power.rrd';
 
         my $monitor = $devdetails->param('CiscoGeneric::power-monitor');
         if( length( $monitor ) > 0 )
