@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoIOS.pm,v 1.10 2007/06/04 15:07:05 ssinyagin Exp $
+# $Id: CiscoIOS.pm,v 1.15 2008/11/29 17:01:27 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Cisco IOS devices discovery
@@ -53,6 +53,7 @@ our %oiddef =
      # CISCO-BGP4-MIB
      'cbgpPeerAddrFamilyName'            => '1.3.6.1.4.1.9.9.187.1.2.3.1.3',
      'cbgpPeerAcceptedPrefixes'          => '1.3.6.1.4.1.9.9.187.1.2.4.1.1',
+     'cbgpPeerPrefixAdminLimit'          => '1.3.6.1.4.1.9.9.187.1.2.4.1.3',
      # CISCO-CAR-MIB
      'ccarConfigTable'                   => '1.3.6.1.4.1.9.9.113.1.1.1',
      'ccarConfigType'                    => '1.3.6.1.4.1.9.9.113.1.1.1.1.3',
@@ -62,6 +63,8 @@ our %oiddef =
      'ccarConfigExtLimit'                => '1.3.6.1.4.1.9.9.113.1.1.1.1.7',
      'ccarConfigConformAction'           => '1.3.6.1.4.1.9.9.113.1.1.1.1.8',
      'ccarConfigExceedAction'            => '1.3.6.1.4.1.9.9.113.1.1.1.1.9',
+     # CISCO-VPDN-MGMT-MIB
+     'cvpdnSystemTunnelTotal'            => '1.3.6.1.4.1.9.10.24.1.1.4.1.2'
      );
 
 
@@ -137,9 +140,25 @@ if( not defined( $interfaceFilter ) )
          'ifType'  => 127,                    # docsCableMaclayer
          'ifDescr' => '^Bundle'
          },
+
+     'EOBCN/N' => {
+         'ifType'  => 53,                     # propVirtual
+         'ifDescr' => '^EOBC'
+         },
+
+     'FIFON/N' => {
+         'ifType'  => 53,                     # propVirtual
+         'ifDescr' => '^FIFO'
+         },
      );
 
-
+our %tunnelType =
+    (
+     # CISCO-VPDN-MGMT-MIB Tunnel Types
+     '1' => 'L2F',
+     '2' => 'L2TP',
+     '3' => 'PPTP'
+    );
 
 
 sub checkdevtype
@@ -304,6 +323,11 @@ sub discover
             $devdetails->storeSnmpVars( $peerTable );
             $devdetails->setCap('CiscoBGP');
 
+            my $limitsTable =
+                $session->get_table( -baseoid =>
+                                     $dd->oiddef('cbgpPeerPrefixAdminLimit') );
+            $limitsTable = {} if not defined( $limitsTable );
+            
             $data->{'cbgpPeers'} = {};
             
             # retrieve AS numbers for neighbor peers
@@ -370,7 +394,11 @@ sub discover
                 }
                 $peer->{'description'} .= '[' . $peerIP . ']';
 
-                $data->{'cbgpPeers'}{$INDEX} = $peer;
+                $peer->{'prefixLimit'} =
+                    $limitsTable->{$dd->oiddef('cbgpPeerPrefixAdminLimit') .
+                                       '.' . $INDEX};
+                    
+                $data->{'cbgpPeers'}{$INDEX} = $peer;                
             }
 
             if( scalar( @nonV4Unicast ) > 0 )
@@ -478,6 +506,43 @@ sub discover
         }
     }
 
+
+    if( $devdetails->param('CiscoIOS::disable-vpdn-stats') ne 'yes' )
+    {
+        if( $dd->checkSnmpTable( 'cvpdnSystemTunnelTotal' ) )
+        {
+            # Find the Tunnel type
+            my $tableTun = $session->get_table(
+                            -baseoid => $dd->oiddef('cvpdnSystemTunnelTotal') );
+
+            if( $tableTun )
+            {
+                $devdetails->setCap("ciscoVPDN");
+
+                $devdetails->storeSnmpVars( $tableTun );
+
+                # VPDN indexing: 1: l2f, 2: l2tp, 3: pptp
+                foreach my $typeIndex (
+                            $devdetails->getSnmpIndices(
+                              $dd->oiddef('cvpdnSystemTunnelTotal') ) )
+                {
+                    Debug("CISCO-VPDN-MGMT-MIB: found Tunnel type " .
+                           $tunnelType{$typeIndex} );
+
+                    $data->{'ciscoVPDN'}{$typeIndex} = $tunnelType{$typeIndex};
+                }
+            }
+        }
+    }
+
+    if( $devdetails->param('CiscoIOS::short-device-comment') eq 'yes' )
+    {
+        # Remove serials from device comment
+        # 1841 chassis, Hw Serial#: 3625140487, Hw Revision: 6.0
+        
+        $data->{'param'}{'comment'} =~ s/, Hw.*//o;        
+    }
+
     return 1;
 }
 
@@ -493,9 +558,8 @@ sub buildConfig
     if( $devdetails->hasCap('CiscoBGP') )
     {
         my $countersNode =
-            $cb->addSubtree( $devNode, 'BGP_Stats',
-                             { 'comment' => 'BGP peer statistics'},
-                             ['CiscoIOS::cisco-bgp-subtree']);
+            $cb->addSubtree( $devNode, 'BGP_Prefixes',
+                             { 'comment' => 'Accepted prefixes'} );
 
         foreach my $INDEX ( sort
                             { $data->{'cbgpPeers'}{$a}{'subtreeName'} <=>
@@ -512,7 +576,14 @@ sub buildConfig
                 'precedence'           => 65000 - $peer->{'peerAS'}
             };
 
-            $cb->addSubtree
+            if( defined( $peer->{'prefixLimit'} ) and
+                $peer->{'prefixLimit'} > 0 )
+            {
+                $param->{'upper-limit'} = $peer->{'prefixLimit'};
+                $param->{'graph-upper-limit'} = $peer->{'prefixLimit'} * 1.03;
+            }
+            
+            $cb->addLeaf
                 ( $countersNode, $peer->{'subtreeName'}, $param,
                   ['CiscoIOS::cisco-bgp'] );
         }
@@ -577,6 +648,25 @@ sub buildConfig
                   ['CiscoIOS::cisco-car']);
         }
     }
+
+
+    if( $devdetails->hasCap('ciscoVPDN') )
+    { 
+        my $tunnelNode = $cb->addSubtree( $devNode, 'VPDN_Statistics',
+                             { 'comment' => 'VPDN statistics' },
+                             [ 'CiscoIOS::cisco-vpdn-subtree' ] );
+
+        foreach my $INDEX ( sort keys %{$data->{'ciscoVPDN'}} )
+        {
+            my $tunnelProtocol = $data->{'ciscoVPDN'}{$INDEX};
+
+            $cb->addSubtree( $tunnelNode, $tunnelProtocol,
+                { 'comment'  => $tunnelProtocol . ' information',
+                  'tunIndex' => $INDEX,
+                  'tunFile'  => lc($tunnelProtocol) },
+                [ 'CiscoIOS::cisco-vpdn-leaf' ] );
+        }
+    }
 }
 
 
