@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RFC2670_DOCS_IF.pm,v 1.13 2007/04/11 15:31:44 ssinyagin Exp $
+# $Id: RFC2670_DOCS_IF.pm,v 1.14 2008/07/23 21:14:20 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # DOCSIS interface statistics
@@ -137,6 +137,12 @@ sub discover
             }
         }
     }
+    
+    if( $devdetails->param('RFC2670_DOCS_IF::upstreams-only') eq 'yes' )
+    {
+        $data->{'docsCableMaclayer'} = [];
+        $data->{'docsCableDownstream'} = [];
+    }
 
     $data->{'docsConfig'} = {
         'docsCableMaclayer' => {
