@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RFC2863_IF_MIB.pm,v 1.33 2007/05/04 11:08:05 ssinyagin Exp $
+# $Id: RFC2863_IF_MIB.pm,v 1.40 2008/07/23 21:14:20 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Standard IF_MIB discovery, which should apply to most devices
@@ -395,11 +395,6 @@ sub buildConfig
 
     uniqueEntries( $devdetails, $data->{'nameref'}{'ifNick'} );
     uniqueEntries( $devdetails, $data->{'nameref'}{'ifSubtreeName'} );
-
-    if( $devdetails->param('RFC2863_IF_MIB::noout') eq 'yes' )
-    {
-        return;
-    }
     
     # Build interface parameters
 
@@ -426,8 +421,9 @@ sub buildConfig
             not defined( $interface->{'param'}{'comment'} ) and
             length( $interface->{$data->{'nameref'}{'ifComment'}} ) > 0 )
         {
-            $interface->{'param'}{'comment'} =
-                $interface->{$data->{'nameref'}{'ifComment'}};
+            my $comment = $interface->{$data->{'nameref'}{'ifComment'}};
+            $interface->{'param'}{'comment'} = $comment;
+            $interface->{'param'}{'interface-comment'} = $comment;
         }
 
         # Order the interfaces by ifIndex, not by interface name
@@ -442,6 +438,11 @@ sub buildConfig
         return;
     }
 
+    if( $devdetails->param('RFC2863_IF_MIB::noout') eq 'yes' )
+    {
+        return;
+    }
+
     # explicitly excluded interfaces    
     my %excludeName;
     my $excludeNameList =
@@ -519,20 +520,51 @@ sub buildConfig
     my $extSrv =
         $devdetails->param('RFC2863_IF_MIB::external-serviceid');
     my %extStorage;
+    my %extStorageTrees;
+    
     if( defined( $extSrv ) and length( $extSrv ) > 0 )
     {
         foreach my $srvDef ( split( /\s*,\s*/, $extSrv ) )
         {
-            my ( $serviceid, $intfName, $direction ) =
+            my ( $serviceid, $intfName, $direction, $trees ) =
                 split( /\s*:\s*/, $srvDef );
-            if( $direction eq 'Both' )
+
+            if( $intfName =~ /\// )
             {
-                $extStorage{$intfName}{'In'} = $serviceid . '_IN';
-                $extStorage{$intfName}{'Out'} = $serviceid . '_OUT';
+                my( $host, $intf ) = split( '/', $intfName );
+                if( $host eq $devdetails->param('snmp-host') )
+                {
+                    $intfName = $intf;
+                }
+                else
+                {
+                    $intfName = undef;
+                }
             }
-            else
+
+            if( defined( $intfName ) and length( $intfName ) > 0 )
             {
-                $extStorage{$intfName}{$direction} = $serviceid;
+                if( defined( $trees ) )
+                {
+                    # Trees are listed with '|' as separator,
+                    # whereas compiler expects commas
+                    
+                    $trees =~ s/\s*\|\s*/,/g;
+                }
+                            
+                if( $direction eq 'Both' )
+                {
+                    $extStorage{$intfName}{'In'} = $serviceid . '_IN';
+                    $extStorageTrees{$serviceid . '_IN'} = $trees;
+                    
+                    $extStorage{$intfName}{'Out'} = $serviceid . '_OUT';
+                    $extStorageTrees{$serviceid . '_OUT'} = $trees;
+                }
+                else
+                {
+                    $extStorage{$intfName}{$direction} = $serviceid;
+                    $extStorageTrees{$serviceid} = $trees;
+                }
             }
         }
     }
@@ -619,9 +651,16 @@ sub buildConfig
     {
         my $interface = $data->{'interfaces'}{$ifIndex};
 
+        if( $interface->{'selectorActions'}{'RemoveInterface'} )
+        {
+            $interface->{'excluded'} = 1;
+            Debug('Removing interface by selector action: ' .
+                  $interface->{$data->{'nameref'}{'ifReferenceName'}});
+        }
+
         # Some vendor-specific modules may exclude some interfaces
         next if $interface->{'excluded'};
-        
+
         # Create a subtree for the interface
         my $subtreeName = $interface->{$data->{'nameref'}{'ifSubtreeName'}};
 
@@ -666,11 +705,25 @@ sub buildConfig
                 if( defined( $interface->{'selectorActions'}->
                              {$dir . 'BytesMonitor'} ) )
                 {
+                    $interface->{'childCustomizations'}->{
+                        'Bytes_' . $dir}->{'monitor'} =
+                            $interface->{'selectorActions'}->{
+                                $dir . 'BytesMonitor'};
+                }
+
+                if( defined( $interface->{'selectorActions'}->
+                             {$dir . 'BytesParameters'} ) )
+                {
+                    my @pairs =
+                        split('\s*;\s*',
+                              $interface->{'selectorActions'}{
+                                  $dir . 'BytesParameters'});
+                    
+                    foreach my $pair( @pairs )
                     {
+                        my ($param, $val) = split('\s*=\s*', $pair);
                         $interface->{'childCustomizations'}->{
-                            'Bytes_' . $dir}->{'monitor'} =
-                                $interface->{'selectorActions'}->{
-                                    $dir . 'BytesMonitor'};
+                            'Bytes_' . $dir}->{$param} = $val;
                     }
                 }
             }
@@ -827,13 +880,22 @@ sub buildConfig
                 {
                     if( defined( $extStorage{$subtreeName}{$dir} ) )
                     {
+                        my $serviceid = $extStorage{$subtreeName}{$dir};
+
+                        my $params = {
+                            'storage-type'      => 'rrd,ext',
+                            'ext-service-id'    => $serviceid,
+                            'ext-service-units' => 'bytes' };
+                        
+                        if( defined( $extStorageTrees{$serviceid} )
+                            and length( $extStorageTrees{$serviceid} ) > 0 )
+                        {
+                            $params->{'ext-service-trees'} =
+                                $extStorageTrees{$serviceid};
+                        }
+                        
                         $interface->{'childCustomizations'}->{
-                            'Bytes_' . $dir}->{'storage-type'} = 'rrd,ext';
-                        $interface->{'childCustomizations'}->{
-                            'Bytes_' . $dir}->{'ext-service-id'} =
-                                $extStorage{$subtreeName}{$dir};
-                        $interface->{'childCustomizations'}->{
-                            'Bytes_' . $dir}->{'ext-service-units'} = 'bytes';
+                            'Bytes_' . $dir} = $params;
                     }
                 }
             }
@@ -1181,8 +1243,11 @@ our %knownSelectorActions =
       'HoltWinters'       => 'RFC2863_IF_MIB',
       'NoPacketCounters'  => 'RFC2863_IF_MIB',
       'NoErrorCounters'   => 'RFC2863_IF_MIB',
+      'RemoveInterface'   => 'RFC2863_IF_MIB',
       'TokensetMember'    => 'RFC2863_IF_MIB',
-      'Parameters'        => 'RFC2863_IF_MIB' );
+      'Parameters'        => 'RFC2863_IF_MIB',
+      'InBytesParameters' => 'RFC2863_IF_MIB',
+      'OutBytesParameters' => 'RFC2863_IF_MIB',);
 
                             
 sub applySelectorAction
@@ -1208,7 +1273,7 @@ sub applySelectorAction
     }
     else
     {
-        Error('Unknown CiscoSensor selector action: ' . $action);
+        Error('Unknown RFC2863_IF_MIB selector action: ' . $action);
     }
 }
    
