@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Renderer.pm,v 1.9 2006/04/18 09:36:51 ssinyagin Exp $
+# $Id: Renderer.pm,v 1.10 2007/04/18 16:05:19 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer;
@@ -80,14 +80,11 @@ sub render
     my $view = shift;
     my %new_options = @_;
 
+    # If no options given, preserve the existing ones
     if( %new_options )
     {
         $self->{'options'} = \%new_options;
     }
-    else
-    {
-        $self->{'options'} = {};
-    }
 
     $self->checkAndClearCache( $config_tree );
 
@@ -144,7 +141,10 @@ sub render
     ($t_expires, $mime_type) =
         $self->$method( $config_tree, $token, $view, $cachefile );
 
-    $self->{'options'} = undef;
+    if( %new_options )
+    {
+        $self->{'options'} = undef;
+    }
 
     my @ret;
     if( defined($t_expires) and defined($mime_type) )
