@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RRDtool.pm,v 1.8 2007/03/23 10:44:53 ssinyagin Exp $
+# $Id: RRDtool.pm,v 1.9 2008/08/07 18:42:12 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer::RRDtool;
@@ -584,7 +584,14 @@ sub rrd_make_opts
     my @args = ();
     foreach my $param ( keys %{$opthash} )
     {
-        my $value = $config_tree->getParam( $view, $param );
+        my $value =
+            $self->{'options'}->{'variables'}->{'G' . $param};
+        
+        if( not defined( $value ) )
+        {
+            $value = $config_tree->getParam( $view, $param );
+        }
+        
         if( defined( $value ) )
         {
             if( ( $param eq 'start' or $param eq 'end' ) and
