@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: MonthlySrvUsage.pm,v 1.8 2007/07/26 09:03:38 ssinyagin Exp $
+# $Id: MonthlySrvUsage.pm,v 1.11 2008/10/25 23:35:22 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # For all service IDs available, build monthly usage figures:
@@ -66,16 +66,20 @@ sub generate
     my $srvIDs = $self->{'srvexport'}->getServiceIDs();
     foreach my $serviceid ( @{$srvIDs} )
     {
+        &Torrus::DB::checkInterrupted();
+        
         my $data = $self->{'srvexport'}->getIntervalData
             ( $self->{'StartDate'}, $self->{'EndDate'}, $serviceid );
 
+        &Torrus::DB::checkInterrupted();
+
         next if scalar( @{$data} ) == 0;
         Debug('MonthlySrvUsage: Generating report for ' . $serviceid);
 
         my $params = $srvIDParams->getParams( $serviceid );
         
         my @aligned = ();
-        $#aligned= floor( $self->{'RangeSeconds'} / $step );
+        $#aligned = floor( $self->{'RangeSeconds'} / $step );
         my $nDatapoints = scalar( @aligned );
             
         # Fill in the aligned array. For each interval by modulo(step),
@@ -106,6 +110,8 @@ sub generate
             }
         }
 
+        &Torrus::DB::checkInterrupted();
+
         # Set undefined values to zero and calculate the average
 
         my $sum = Math::BigFloat->new(0);
@@ -123,6 +129,8 @@ sub generate
             }
         }
 
+        &Torrus::DB::checkInterrupted();
+
         my $avgVal = $sum / $nDatapoints;
 
         # Calculate the percentile
@@ -131,11 +139,11 @@ sub generate
         my $pcPos = floor( $nDatapoints * $percentile / 100 );
         my $pcVal = $sorted[$pcPos];
 
-        # Calculate the total volume, if it's a counter
+        # Calculate the total volume if it's a counter
         my $volume = Math::BigFloat->new(0);
         my $volumeDefined = 0;
         if( not defined( $params->{'dstype'} ) or
-            $params->{'dstype'} =~ /^COUNTER/ )
+            $params->{'dstype'} =~ /^COUNTER/o )
         {
             $volumeDefined = 1;
             foreach my $row ( @{$data} )
@@ -196,6 +204,8 @@ sub generate
         }
     }
 
+    &Torrus::DB::checkInterrupted();
+
     $self->{'backend'}->finalize( $self->{'reportId'} );
 }
 
