@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: HTML.pm,v 1.6 2007/06/14 15:43:13 ssinyagin Exp $
+# $Id: HTML.pm,v 1.7 2007/12/09 18:05:25 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::ReportOutput::HTML;
@@ -214,9 +214,15 @@ sub render
         'yearlyUrl' => sub {
             return $self->reportUrl($self->yearlyFilename($_[0]));},
         'monthName' => sub {$self->monthName($_[0]);},
-        'formatValue' => sub {return sprintf('%.2f %s',
-                                             $_[0]->{'value'},
-                                             $_[0]->{'units'})},
+        'formatValue' => sub {
+            if( ref($_[0]))
+            {
+                return sprintf('%.2f %s', $_[0]->{'value'}, $_[0]->{'units'});
+            }
+            else
+            {
+                return 'N/A';
+            }},
         'timestamp'  => sub { return time2str($Torrus::Renderer::timeFormat,
                                               time()); },
     };
